/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 10/12/22.
//

#include "hexboard/scene.hpp"

namespace demo::hexboard {

	Scene::Scene(fggl::App &app) : GameBase(app), m_board(nullptr) {
	}

	void Scene::activate() {
		m_board = std::make_unique<fggl::grid::HexGrid>();
	}

	void Scene::deactivate() {
		m_board = nullptr;
	}

	void Scene::update(float delta) {
		GameBase::update(delta);

		// if the board is not set, abort
		if ( m_board == nullptr ){
			return;
		}
	}

	// see https://www.redblobgames.com/grids/hexagons/#hex-to-pixel
	const fggl::math::mat2 HEX_BASIS{
		std::sqrt(3.0F), std::sqrt(3.0F) / 2.0F,
		0.0F, 3.0F / 2.0F
	};

	static inline fggl::math::vec2 hexToScreen(fggl::grid::IntHex hexPos, float size, fggl::math::vec2 offset = {0,0}) {
		return size * fggl::math::vec2{hexPos.q(), hexPos.r()} * HEX_BASIS + offset;
	}

	void Scene::render(fggl::gfx::Graphics &gfx) {
		// if the board is not set, abort
		if ( m_board == nullptr ){
			return;
		}

		// draw the grid
		// FIXME don't hard-code the screen size
		const float hexRadius = 64.0F;
		const auto gridWidth = (int)( (1920 - hexRadius) / (hexRadius * std::sqrt(3.0F)) );
		const auto gridHeight = (int)( (1080 - hexRadius) / (hexRadius * (3.0F / 2.0F) ));

		const fggl::math::vec2 offset{
			( 1920 - ( (float)gridWidth * hexRadius * std::sqrt(3.0F))) - (hexRadius / 2.0F),
			( (1080 - hexRadius / 2.0F) - ( (float)gridHeight * hexRadius * (3.0F / 2.0F) )),
		};

		fggl::gfx::Paint paint;
		fggl::grid::IntHex hexPos{0, 0};
		auto rowBasis = hexPos;
		for(auto i=0; i<gridHeight; ++i) {
			for (auto j=0; j<gridWidth; ++j) {

				if ( m_board->isValidPos(hexPos) ) {
					auto pos = hexToScreen(hexPos, hexRadius, offset);
					auto hexShape = fggl::gfx::make_shape(pos, hexRadius, 6);
					paint.stroke(hexShape);
				}

				// next hexagon
				hexPos = hexPos.neighbour(fggl::grid::HexDirPointy::RIGHT);
			}

			rowBasis = i % 2 == 0 ? rowBasis.neighbour(fggl::grid::HexDirPointy::BOTTOM_RIGHT) : rowBasis.neighbour(fggl::grid::HexDirPointy::BOTTOM_LEFT);
			hexPos = rowBasis;
		}

		gfx.draw2D(paint);
	}



} // namespace demo::hexboard